package ic2backpackhud;

import ic2.api.item.IElectricItem;
import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityClientPlayerMP;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.ScaledResolution;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.RenderItem;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.util.ResourceLocation;
import org.lwjgl.opengl.GL11;
import cpw.mods.fml.client.FMLClientHandler;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.common.gameevent.TickEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public class ClientTickHandler
{
    private Minecraft minecraft = FMLClientHandler.instance().getClient();
    private RenderItem itemRenderer = new RenderItem();
    int scaledWidth, scaledHeight;
    long[] elapsedDrawTime = new long[4];
    int[] currentCharge = new int[4], lastCharge = new int[4];
    boolean[] isCharging = new boolean[4], isLowPower = new boolean[4];
    final byte pixelNudge = 1;
    final byte bgHeight = 21, bgWidth = 18;
    public float zLevel = 0.0F;

    private static final ResourceLocation hudTextures = new ResourceLocation("IC2BackpackHUD:textures/sprites.png");
    private static final ResourceLocation glintEffect = new ResourceLocation("textures/misc/enchanted_item_glint.png");

    private final byte POS_HORIZ_LEFT  = 0;
    private final byte POS_HORIZ_RIGHT = 1;
    private final byte POS_VERTICAL_TL = 2;
    private final byte POS_VERTICAL_BL = 3;
    private final byte POS_VERTICAL_BR = 4;
    private final byte POS_VERTICAL_TR = 5;

    @SubscribeEvent
    public void tickRender(TickEvent.RenderTickEvent event)
    {
        EntityClientPlayerMP player = minecraft.thePlayer;

        if (Minecraft.getMinecraft().currentScreen != null || player == null) {
            return;
        }

        ScaledResolution scaledResolution = new ScaledResolution(minecraft, minecraft.displayWidth, minecraft.displayHeight);
        scaledWidth = scaledResolution.getScaledWidth();
        scaledHeight = scaledResolution.getScaledHeight();

        /**
         * Armor Inventory Key
         *
         * 	Helmet 		= armorInventory[3]
         * 	Chestplate 	= armorInventory[2]
         * 	Leggings 	= armorInventory[1]
         * 	Boots 		= armorInventory[0]
         */

        ItemStack[] armorInventory = player.inventory.armorInventory;
        boolean[] isValidArmor = new boolean[4];
        boolean hasValidArmor = false;
        byte armorPiecesWorn = 0;

        /* Detect worn armor pieces. */

        for (int idx = 0; idx < 4; idx++) {
            if (armorInventory[idx] != null) {
                if (IC2BackpackHUD.ic2Loaded && armorInventory[idx].getItem() instanceof IElectricItem || IC2BackpackHUD.showNonElectricItems && armorInventory[idx].getItem().isDamageable() && armorInventory[idx].getItemDamage() >= 0) {
                    armorPiecesWorn++;
                    isValidArmor[idx] = true;
                    hasValidArmor = true;
                }
            }
        }

        if (hasValidArmor) {

            int[] xCoord = new int[4];
            int[] yCoord = new int[4];

            for (byte count = 3, tempPiecesWorn = armorPiecesWorn; count >= 0; count--) {

                if (isValidArmor[count]) {

                    switch(IC2BackpackHUD.hudPosition) {
                    case POS_VERTICAL_TL:
                        xCoord[count] = pixelNudge * 3;
                        yCoord[count] = pixelNudge - 16 + (((armorPiecesWorn + 1) - tempPiecesWorn--) * bgHeight);
                        break;
                    case POS_VERTICAL_BL:
                        xCoord[count] = pixelNudge * 3;
                        yCoord[count] = scaledHeight - 16 - (pixelNudge * 3) - ((tempPiecesWorn-- - 1) * bgHeight);
                        break;
                    case POS_HORIZ_LEFT:
                        xCoord[count] = (scaledWidth / 2 - 92) - tempPiecesWorn-- * bgWidth;
                        yCoord[count] = scaledHeight - 16 - pixelNudge * 2;
                        break;
                    case POS_HORIZ_RIGHT:
                        xCoord[count] = (scaledWidth / 2 + 94) + (armorPiecesWorn - tempPiecesWorn--) * bgWidth;
                        yCoord[count] = scaledHeight - 16 - pixelNudge * 2;
                        break;
                    case POS_VERTICAL_BR:
                        xCoord[count] = scaledWidth - pixelNudge - bgWidth;
                        yCoord[count] = scaledHeight - 16 - (pixelNudge * 3) - ((tempPiecesWorn-- - 1) * bgHeight);
                        break;
                    case POS_VERTICAL_TR:
                        xCoord[count] = scaledWidth - pixelNudge - bgWidth;
                        yCoord[count] = pixelNudge - 16 + (((armorPiecesWorn + 1) - tempPiecesWorn--) * bgHeight);
                        break;
                    }

                }

            }

            /* Draw armor pieces. */

            if (isValidArmor[3] && IC2BackpackHUD.showHelmetItems) {
                drawArmor(3, xCoord[3], yCoord[3]);
            }
            if (isValidArmor[2] && IC2BackpackHUD.showChestplateItems) {
                drawArmor(2, xCoord[2], yCoord[2]);
            }
            if (isValidArmor[1] && IC2BackpackHUD.showLeggingItems) {
                drawArmor(1, xCoord[1], yCoord[1]);
            }
            if (isValidArmor[0] && IC2BackpackHUD.showBootItems) {
                drawArmor(0, xCoord[0], yCoord[0]);
            }

        }

    }

    /**
     * Return ItemStack damage as normalized integer between 0 and 14
     */
    int getChargeLevel(ItemStack armor)
    {
        int defaultCharge = (int) Math.round(13.0D - armor.getItemDamageForDisplay() * 13.0D / armor.getMaxDamage());

        if (armor.isItemDamaged()) {
            /*
             * Vanilla item damage level is computed using a 13-value range.  This introduces rounding
             * errors for low max damage items when scaling to a 15-value range.  This returns the
             * correct zero-value item charge according to default item damage level checks.
             */
            if (defaultCharge == 0) {
                return 0;
            } else {
                return (int) Math.round(15.0D - armor.getItemDamageForDisplay() * 15.0D / armor.getMaxDamage());
            }
        } else {
            return 14;
        }
    }

    /**
     * Draws a textured rectangle at the stored z-value. Args: x, y, u, v
     */
    public void renderIconUsingOffset(int x, int y, int z, int uOffset, int vOffset, int u, int v)
    {
        float factor = 0.00390625F;
        Tessellator tessellator = Tessellator.instance;
        tessellator.startDrawingQuads();
        tessellator.addVertexWithUV(x + 0, y + v, z, (uOffset + 0) * factor, (vOffset + v) * factor);
        tessellator.addVertexWithUV(x + u, y + v, z, (uOffset + u) * factor, (vOffset + v) * factor);
        tessellator.addVertexWithUV(x + u, y + 0, z, (uOffset + u) * factor, (vOffset + 0) * factor);
        tessellator.addVertexWithUV(x + 0, y + 0, z, (uOffset + 0) * factor, (vOffset + 0) * factor);
        tessellator.draw();
    }

    /**
     * Renders icon.
     */
    public void renderIcon(int x, int y, IIcon icon, int width, int height)
    {
        Tessellator tessellator = Tessellator.instance;
        tessellator.startDrawingQuads();
        tessellator.addVertexWithUV(x + 0, y + height, zLevel, icon.getMinU(), icon.getMaxV());
        tessellator.addVertexWithUV(x + width, y + height, zLevel, icon.getMaxU(), icon.getMaxV());
        tessellator.addVertexWithUV(x + width, y + 0, zLevel, icon.getMaxU(), icon.getMinV());
        tessellator.addVertexWithUV(x + 0, y + 0, zLevel, icon.getMinU(), icon.getMinV());
        tessellator.draw();
    }

    /**
     * Draw armor item starting at given coordinate
     */
    void drawArmor(int armorNum, int xCoord, int yCoord)
    {
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

        ItemStack itemStack = minecraft.thePlayer.inventory.armorInventory[armorNum];
        int chargeLevel = getChargeLevel(itemStack);
        float dura_red = 1.0F - chargeLevel / 14.0F;
        float dura_grn = chargeLevel / 14.0F;

        /* Draw status background */

        if (IC2BackpackHUD.showBackground) {
            minecraft.renderEngine.bindTexture(hudTextures);
            GL11.glColor4f(isCharging[armorNum] ? 1.0F : dura_red, isCharging[armorNum] ? 1.0F : dura_grn, 0.0F, 1.0F);
            renderIconUsingOffset(xCoord - 1, yCoord - 4, -100, 16, 0, bgWidth, bgHeight);
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        }

        /* Draw item */

        minecraft.renderEngine.bindTexture(TextureMap.locationItemsTexture);
        renderItemAndEffectIntoGUI(minecraft.fontRenderer, minecraft.renderEngine, itemStack, xCoord, yCoord - 2);

        /* Draw status icons */

        minecraft.renderEngine.bindTexture(hudTextures);

        int elapsedTime = (int) (System.nanoTime() - elapsedDrawTime[armorNum]);
        if (elapsedTime > 500000000) {

            isCharging[armorNum] = false;
            isLowPower[armorNum] = false;

            if (IC2BackpackHUD.ic2Loaded && IC2BackpackHUD.showChargingFlasher) {
                if (itemStack.getItem() instanceof IElectricItem) {

                    if (itemStack.stackTagCompound != null && itemStack.stackTagCompound.hasKey("charge")) {
                        currentCharge[armorNum] = itemStack.stackTagCompound.getInteger("charge");
                    }

                    if (currentCharge[armorNum] > lastCharge[armorNum]) {
                        isCharging[armorNum] = true;
                    }

                    lastCharge[armorNum] = currentCharge[armorNum];
                }
            }

            if (IC2BackpackHUD.showLowPowerFlasher) {
                if (chargeLevel <= 2) {
                    isLowPower[armorNum] = true;
                }
            }

            elapsedDrawTime[armorNum] = System.nanoTime();
        }

        if (isLowPower[armorNum] || isCharging[armorNum]) {
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F - (chargeLevel == 0 ? 0 : (Math.abs(elapsedTime - 250000000) / 250000000.0F)));
            renderIconUsingOffset(xCoord, yCoord - 3, 0, 0, (isCharging[armorNum] ? 16 : 0), 16, 16);
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        }

        /* Draw durability or charge bar */

        GL11.glDisable(GL11.GL_TEXTURE_2D);
        GL11.glBegin(GL11.GL_QUADS);

        // Charge background
        GL11.glColor4f(0.03515625F, 0.0625F, 0.046875F, 1.0F);
        GL11.glVertex3d(xCoord + 1, yCoord + 13, 0);
        GL11.glVertex3d(xCoord + 1, yCoord + 13 + 2, 0);
        GL11.glVertex3d(xCoord + 15, yCoord + 13 + 2, 0);
        GL11.glVertex3d(xCoord + 15, yCoord + 13, 0);

        // Depleted portion
        GL11.glColor4f(0.15625F, 0.24609375F, 0, 1.0F);
        GL11.glVertex3d(xCoord + 1, yCoord + 13, 0);
        GL11.glVertex3d(xCoord + 1, yCoord + 13 + 1, 0);
        GL11.glVertex3d(xCoord + 14, yCoord + 13 + 1, 0);
        GL11.glVertex3d(xCoord + 14, yCoord + 13, 0);

        // Charged portion
        GL11.glColor4f(isCharging[armorNum] ? 1.0F : dura_red, isCharging[armorNum] ? 1.0F : dura_grn, 0.0F, 1.0F);
        GL11.glVertex3d(xCoord + 1, yCoord + 13, 0);
        GL11.glVertex3d(xCoord + 1, yCoord + 13 + 1, 0);
        GL11.glVertex3d(xCoord + 1 + chargeLevel, yCoord + 13 + 1, 0);
        GL11.glVertex3d(xCoord + 1 + chargeLevel, yCoord + 13, 0);
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);

        GL11.glEnd();
        GL11.glEnable(GL11.GL_TEXTURE_2D);

        GL11.glDisable(GL11.GL_BLEND);
    }

    /**
     * Returns integer color as rgb[] float array.
     */
    private float[] getRGB(int color)
    {
        float[] rgb = {(color >> 16 & 255) / 255.0F, (color >> 8 & 255) / 255.0F, (color & 255) / 255.0F };
        return rgb;
    }

    /**
     * Renders the item's icon or block into the UI at the specified position.
     */
    public void renderItemAndEffectIntoGUI(FontRenderer fontRenderer, TextureManager textureManager, ItemStack itemStack, int x, int y)
    {
        Item item = itemStack.getItem();
        IIcon icon = itemStack.getIconIndex();
        if (icon == null) {

            TextureManager texturemanager = Minecraft.getMinecraft().getTextureManager();
            ResourceLocation resourcelocation = texturemanager.getResourceLocation(itemStack.getItemSpriteNumber());
            icon = ((TextureMap)texturemanager.getTexture(resourcelocation)).getAtlasSprite("missingno");

        }

        if (item.requiresMultipleRenderPasses() && icon != null) {

            for (int pass = 0; pass < item.getRenderPasses(itemStack.getItemDamage()); ++pass) {
                IIcon iconForPass = item.getIcon(itemStack, pass);
                float[] rgb = getRGB(item.getColorFromItemStack(itemStack, pass));
                GL11.glColor4f(rgb[0], rgb[1], rgb[2], 1.0F);
                renderIcon(x, y, iconForPass, 16, 16);
            }

        } else {

            float[] rgb = getRGB(item.getColorFromItemStack(itemStack, 0));
            GL11.glColor4f(rgb[0], rgb[1], rgb[2], 1.0F);
            renderIcon(x, y, icon, 16, 16);

        }

        if (itemStack.hasEffect(0))
        {
            GL11.glDepthFunc(GL11.GL_GREATER);
            GL11.glDepthMask(false);
            textureManager.bindTexture(glintEffect);
            zLevel -= 50.0F;
            GL11.glBlendFunc(GL11.GL_DST_COLOR, GL11.GL_DST_COLOR);
            GL11.glColor4f(0.5F, 0.25F, 0.8F, 1.0F);
            renderGlint(x - 2, y - 2, 20, 20);
            GL11.glDepthMask(true);
            zLevel += 50.0F;
            GL11.glDepthFunc(GL11.GL_LEQUAL);
            GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        }
    }

    /**
     * Renders glint effect.
     */
    private void renderGlint(int x, int y, int width, int height)
    {
        for (int pass = 0; pass < 2; ++pass)
        {
            GL11.glBlendFunc(GL11.GL_SRC_COLOR, GL11.GL_ONE);

            float vFactor = 0.00390625F;
            float timeOffset = Minecraft.getSystemTime() % (3000 + pass * 1873) / (3000.0F + pass * 1873) * 256.0F;
            Tessellator tessellator = Tessellator.instance;
            float uMultiple = 4.0F;

            if (pass == 1) {
                uMultiple = -1.0F;
            }

            tessellator.startDrawingQuads();
            tessellator.addVertexWithUV(x + 0, y + height, zLevel, (timeOffset + height * uMultiple) * vFactor, height * vFactor);
            tessellator.addVertexWithUV(x + width, y + height, zLevel, (timeOffset + width + height * uMultiple) * vFactor, height * vFactor);
            tessellator.addVertexWithUV(x + width, y + 0, zLevel, (timeOffset + width) * vFactor, 0.0D);
            tessellator.addVertexWithUV(x + 0, y + 0, zLevel, (timeOffset + 0.0F) * vFactor, 0.0D);
            tessellator.draw();
        }
    }

}
